@@ -12,6 +12,7 @@
 
 #include "quill/backend/BackendOptions.h"
 #include "quill/backend/BackendUtilities.h"
+#include "quill/backend/BackendWorkerLock.h"
 #include "quill/backend/BacktraceStorage.h"
 #include "quill/backend/PatternFormatter.h"
 #include "quill/backend/RdtscClock.h"
@@ -21,6 +22,7 @@
 
 #include "quill/core/Attributes.h"
 #include "quill/core/BoundedSPSCQueue.h"
+#include "quill/core/ChronoTimeUtils.h"
 #include "quill/core/Codec.h"
 #include "quill/core/Common.h"
 #include "quill/core/DynamicFormatArgStore.h"
@@ -89,15 +91,14 @@ public:
     // This destructor will run during static destruction as the thread is part of the singleton
     stop();
 
-    RdtscClock const* rdtsc_clock{_rdtsc_clock.load()};
-    _rdtsc_clock.store(nullptr);
+    RdtscClock const* rdtsc_clock = _rdtsc_clock.exchange(nullptr);
     delete rdtsc_clock;
   }
 
   /***/
   QUILL_NODISCARD bool is_running() const noexcept
   {
-    return _is_worker_running.load(std::memory_order_relaxed);
+    return _is_worker_running.load(std::memory_order_acquire);
   }
 
   /**
@@ -133,8 +134,13 @@ public:
   {
     _ensure_linker_retains_symbols();
 
+    if (options.check_backend_singleton_instance)
+    {
+      _backend_worker_lock = std::make_unique<BackendWorkerLock>(_process_id);
+    }
+
     std::thread worker(
-      [this, &options]()
+      [this, options]()
       {
         _init(options);
 
@@ -219,6 +225,9 @@ public:
     {
       _worker_thread.join();
     }
+
+    _worker_thread_id.store(0);
+    _backend_worker_lock.reset(nullptr);
   }
 
   /**
@@ -310,6 +319,7 @@ private:
       {
         _cleanup_invalidated_thread_contexts();
         _cleanup_invalidated_loggers();
+        _try_shrink_empty_transit_event_buffers();
 
         // There is nothing left to do, and we can let this thread sleep for a while
         // buffer events are 0 here and also all the producer queues are empty
@@ -408,6 +418,9 @@ private:
         }
       }
     }
+
+    _cleanup_invalidated_thread_contexts();
+    _cleanup_invalidated_loggers();
   }
 
   /**
@@ -416,9 +429,7 @@ private:
   QUILL_ATTRIBUTE_HOT size_t _populate_transit_events_from_frontend_queues()
   {
     uint64_t const ts_now = _options.log_timestamp_ordering_grace_period.count()
-      ? static_cast<uint64_t>((std::chrono::duration_cast<std::chrono::nanoseconds>(
-                                 std::chrono::system_clock::now().time_since_epoch()) -
-                               _options.log_timestamp_ordering_grace_period)
+      ? static_cast<uint64_t>((detail::get_timestamp<std::chrono::system_clock>() - _options.log_timestamp_ordering_grace_period)
                                 .count())
       : std::numeric_limits<uint64_t>::max();
 
@@ -517,6 +528,8 @@ private:
     // Allocate a new TransitEvent or use an existing one to store the message from the queue
     TransitEvent* transit_event = thread_context->_transit_event_buffer->back();
 
+    assert(transit_event->formatted_msg);
+
     std::memcpy(&transit_event->timestamp, read_pos, sizeof(transit_event->timestamp));
     read_pos += sizeof(transit_event->timestamp);
 
@@ -585,13 +598,19 @@ private:
     read_pos += sizeof(format_args_decoder);
 
     // we need to check and do not try to format the flush events as that wouldn't be valid
-    if (transit_event->macro_metadata->event() != MacroMetadata::Event::Flush)
+    if ((transit_event->macro_metadata->event() != MacroMetadata::Event::Flush) &&
+        (transit_event->macro_metadata->event() != MacroMetadata::Event::LoggerRemovalRequest))
     {
       format_args_decoder(read_pos, _format_args_store);
 
       if (!transit_event->macro_metadata->has_named_args())
       {
         _populate_formatted_log_message(transit_event, transit_event->macro_metadata->message_format());
+
+        if (transit_event->macro_metadata->event() == MacroMetadata::Event::LogWithRuntimeMetadata)
+        {
+          _apply_runtime_metadata(transit_event);
+        }
       }
       else
       {
@@ -626,7 +645,7 @@ private:
         }
       }
     }
-    else
+    else if (transit_event->macro_metadata->event() == MacroMetadata::Event::Flush)
     {
       // if this is a flush event then we do not need to format anything for the
       // transit_event, but we need to set the transit event's flush_flag pointer instead
@@ -635,6 +654,19 @@ private:
       transit_event->flush_flag = reinterpret_cast<std::atomic<bool>*>(flush_flag_tmp);
       read_pos += sizeof(uintptr_t);
     }
+    else
+    {
+      // Store the logger name and the sync flag
+      assert(transit_event->macro_metadata->event() == MacroMetadata::Event::LoggerRemovalRequest);
+
+      uintptr_t logger_removal_flag_tmp;
+      std::memcpy(&logger_removal_flag_tmp, read_pos, sizeof(uintptr_t));
+      read_pos += sizeof(uintptr_t);
+      std::string_view const logger_name = Codec<std::string>::decode_arg(read_pos);
+
+      _logger_removal_flags.emplace(std::string{logger_name},
+                                    reinterpret_cast<std::atomic<bool>*>(logger_removal_flag_tmp));
+    }
 
     if (transit_event->macro_metadata->log_level() == LogLevel::Dynamic)
     {
@@ -798,9 +830,14 @@ private:
       {
         if (transit_event.logger_base->backtrace_storage)
         {
-          // this is a backtrace log and we will store it
+          // this is a backtrace log and we will store the transit event
+          // we need to pass a copy of transit_event here and not move the existing
+          // the transit events are reused
+          TransitEvent transit_event_copy;
+          transit_event.copy_to(transit_event_copy);
+
           transit_event.logger_base->backtrace_storage->store(
-            std::move(transit_event), thread_context.thread_id(), thread_context.thread_name());
+            std::move(transit_event_copy), thread_context.thread_id(), thread_context.thread_name());
         }
         else
         {
@@ -902,8 +939,9 @@ private:
     {
       // if the log_message ends with \n we should exclude it
       size_t const log_message_size =
-        transit_event.formatted_msg->data()[transit_event.formatted_msg->size() - 1] == '\n'
-        ? transit_event.formatted_msg->size() - 2
+        ((transit_event.formatted_msg->size() > 0) &&
+         (transit_event.formatted_msg->data()[transit_event.formatted_msg->size() - 1] == '\n'))
+        ? transit_event.formatted_msg->size() - 1
         : transit_event.formatted_msg->size();
 
       // process the whole message without adding metadata to each line
@@ -924,6 +962,14 @@ private:
     auto const msg =
       std::string_view{transit_event.formatted_msg->data(), transit_event.formatted_msg->size()};
 
+    if (QUILL_UNLIKELY(msg.empty()))
+    {
+      // Process an empty message
+      _write_log_statement(transit_event, thread_id, thread_name, log_level_description,
+                           log_level_short_code, msg);
+      return;
+    }
+
     size_t start = 0;
     while (start < msg.size())
     {
@@ -965,10 +1011,28 @@ private:
                                   thread_name, transit_event.logger_base->logger_name,
                                   transit_event.log_level(), log_message, log_statement))
       {
+        std::string_view log_to_write = log_statement;
+
+        // If the sink has an override pattern formatter to use, prepare the override formatted statement
+        if (sink->_override_pattern_formatter_options)
+        {
+          if (!sink->_override_pattern_formatter)
+          {
+            sink->_override_pattern_formatter =
+              std::make_shared<PatternFormatter>(*sink->_override_pattern_formatter_options);
+          }
+
+          log_to_write = sink->_override_pattern_formatter->format(
+            transit_event.timestamp, thread_id, thread_name, _process_id,
+            transit_event.logger_base->logger_name, log_level_description, log_level_short_code,
+            *transit_event.macro_metadata, transit_event.named_args.get(), log_message);
+        }
+
+        // Forward the message using the computed log statement
         sink->write_log(transit_event.macro_metadata, transit_event.timestamp, thread_id,
                         thread_name, _process_id, transit_event.logger_base->logger_name,
                         transit_event.log_level(), log_level_description, log_level_short_code,
-                        transit_event.named_args.get(), log_message, log_statement);
+                        transit_event.named_args.get(), log_message, log_to_write);
       }
     }
   }
@@ -1106,6 +1170,13 @@ private:
 
     if (read_result.allocation)
     {
+      if ((read_result.new_capacity < read_result.previous_capacity) && thread_context->_transit_event_buffer)
+      {
+        // The user explicitly requested to shrink the queue, indicating a preference for low memory
+        // usage. To align with this intent, we also request shrinking the backend buffer.
+        thread_context->_transit_event_buffer->request_shrink();
+      }
+
       // When allocation_info has a value it means that the queue has re-allocated
       if (_options.error_notifier)
       {
@@ -1358,6 +1429,32 @@ private:
       // if loggers were removed also check for sinks to remove
       // cleanup_unused_sinks is expensive and should be only called when it is needed
       _sink_manager.cleanup_unused_sinks();
+
+      for (auto const& removed_logger_name : removed_loggers)
+      {
+        // Notify the user if the blocking call was used
+        auto search_it = _logger_removal_flags.find(removed_logger_name);
+        if (search_it != _logger_removal_flags.end())
+        {
+          search_it->second->store(true);
+          _logger_removal_flags.erase(search_it);
+        }
+      }
+    }
+  }
+
+  /**
+   * Shrinks empty TransitEvent buffers. This is triggered only when the user explicitly
+   * requests shrinking of the unbounded frontend queue to optimize memory usage.
+   */
+  QUILL_ATTRIBUTE_HOT void _try_shrink_empty_transit_event_buffers()
+  {
+    for (ThreadContext* thread_context : _active_thread_contexts_cache)
+    {
+      if (thread_context->_transit_event_buffer)
+      {
+        thread_context->_transit_event_buffer->try_shrink();
+      }
     }
   }
 
@@ -1374,7 +1471,7 @@ private:
   {
     // Generate a format string
     std::string format_string;
-    static constexpr std::string_view delimiter{"\x01\x02\x03"};
+    static constexpr std::string_view delimiter{QUILL_MAGIC_SEPARATOR};
 
     for (size_t i = 0; i < named_args.size(); ++i)
     {
@@ -1483,8 +1580,10 @@ private:
                            fmtquill::basic_format_args<fmtquill::format_context>{
                              _format_args_store.data(), _format_args_store.size()});
 
-      if (_options.check_printable_char && _format_args_store.has_string_related_type())
+      if (_options.check_printable_char && _format_args_store.has_string_related_type() &&
+          (transit_event->macro_metadata->event() != MacroMetadata::Event::LogWithRuntimeMetadata))
       {
+        // we do not want to sanitise LogWithRuntimeMetadata yet because it includes a special separator
         // if non-printable chars check is configured or if any of the provided arguments are strings
         sanitize_non_printable_chars(*transit_event->formatted_msg, _options);
       }
@@ -1504,6 +1603,58 @@ private:
 #endif
   }
 
+  void _apply_runtime_metadata(TransitEvent* transit_event)
+  {
+    // Create a view over the complete formatted log message.
+    // Format: file, line, function, message (separated by QUILL_MAGIC_SEPARATOR).
+    auto const formatted_view =
+      std::string_view{transit_event->formatted_msg->data(), transit_event->formatted_msg->size()};
+
+    static constexpr std::string_view delimiter{QUILL_MAGIC_SEPARATOR};
+
+    // Find the positions of the first three delimiters.
+    auto const pos_first_delim = formatted_view.find(delimiter);
+    auto const pos_second_delim = formatted_view.find(delimiter, pos_first_delim + delimiter.size());
+    auto const pos_third_delim = formatted_view.find(delimiter, pos_second_delim + delimiter.size());
+
+    // Extract the four components
+    std::string_view message = formatted_view.substr(0, pos_first_delim);
+    std::string_view file = formatted_view.substr(
+      pos_first_delim + delimiter.size(), pos_second_delim - pos_first_delim - delimiter.size());
+    std::string_view line = formatted_view.substr(
+      pos_second_delim + delimiter.size(), pos_third_delim - pos_second_delim - delimiter.size());
+    std::string_view function_name = formatted_view.substr(pos_third_delim + delimiter.size());
+
+    std::string const fileline = std::string{file} + ":" + std::string{line};
+
+    // Use fileline and function_name as a composite key for runtime metadata lookup.
+    std::pair<std::string, std::string> const metadata_key =
+      std::make_pair(fileline, std::string{function_name});
+
+    // Use existing metadata if available; otherwise, create new metadata.
+    if (auto search_it = _runtime_metadata.find(metadata_key); search_it != _runtime_metadata.end())
+    {
+      transit_event->macro_metadata = search_it->second.get();
+    }
+    else
+    {
+      auto [it, inserted] = _runtime_metadata.emplace(metadata_key, nullptr);
+      it->second = std::make_unique<MacroMetadata>(
+        it->first.first.data() /* fileline */, it->first.second.data() /* function_name */, "{}",
+        nullptr, LogLevel::Dynamic, MacroMetadata::Event::Log);
+      transit_event->macro_metadata = it->second.get();
+    }
+
+    // Resize the formatted message to retain only the log message
+    transit_event->formatted_msg->try_resize(message.size());
+
+    if (_options.check_printable_char)
+    {
+      // sanitize non-printable characters if enabled.
+      sanitize_non_printable_chars(*transit_event->formatted_msg, _options);
+    }
+  }
+
   template <typename TFormattedMsg>
   static void sanitize_non_printable_chars(TFormattedMsg& formatted_msg, BackendOptions const& options)
   {
@@ -1545,8 +1696,20 @@ private:
   }
 
 private:
+  struct PairHash
+  {
+    std::size_t operator()(const std::pair<std::string, std::string>& p) const
+    {
+      auto h1 = std::hash<std::string>{}(p.first);
+      auto h2 = std::hash<std::string>{}(p.second);
+      // Combine the two hashes.
+      return h1 ^ (h2 << 1);
+    }
+  };
+
   friend class quill::ManualBackendWorker;
 
+  std::unique_ptr<BackendWorkerLock> _backend_worker_lock;
   ThreadContextManager& _thread_context_manager = ThreadContextManager::instance();
   SinkManager& _sink_manager = SinkManager::instance();
   LoggerManager& _logger_manager = LoggerManager::instance();
@@ -1557,7 +1720,8 @@ private:
   std::vector<ThreadContext*> _active_thread_contexts_cache;
   std::vector<Sink*> _active_sinks_cache; /** Member to avoid re-allocating **/
   std::unordered_map<std::string, std::pair<std::string, std::vector<std::pair<std::string, std::string>>>> _named_args_templates; /** Avoid re-formating the same named args log template each time */
-
+  std::unordered_map<std::pair<std::string, std::string>, std::unique_ptr<MacroMetadata>, PairHash> _runtime_metadata; /** Used to store runtime metadata **/
+  std::unordered_map<std::string, std::atomic<bool>*> _logger_removal_flags; /** Maps logger names to atomic flags used for synchronizing remove_logger_blocking(). */
   std::string _named_args_format_template; /** to avoid allocation each time **/
   std::string _process_id;                 /** Id of the current running process **/
   std::chrono::steady_clock::time_point _last_rdtsc_resync_time;
@@ -1565,9 +1729,9 @@ private:
   std::atomic<uint32_t> _worker_thread_id{0};  /** cached backend worker thread id */
   std::atomic<bool> _is_worker_running{false}; /** The spawned backend thread status */
 
-  alignas(CACHE_LINE_ALIGNED) std::atomic<RdtscClock*> _rdtsc_clock{
+  alignas(QUILL_CACHE_LINE_ALIGNED) std::atomic<RdtscClock*> _rdtsc_clock{
     nullptr}; /** rdtsc clock if enabled, can be accessed by any thread **/
-  alignas(CACHE_LINE_ALIGNED) std::mutex _wake_up_mutex;
+  alignas(QUILL_CACHE_LINE_ALIGNED) std::mutex _wake_up_mutex;
   std::condition_variable _wake_up_cv;
   bool _wake_up_flag{false};
 };
